@@ -1,5 +1,7 @@
-from __future__ import unicode_literals
+from typing import Sequence
+from typing import Tuple
 
+from pre_commit.hook import Hook
 from pre_commit.languages import helpers
 
 
@@ -9,5 +11,9 @@ healthy = helpers.basic_healthy
 install_environment = helpers.no_install
 
 
-def run_hook(hook, file_args, color):
+def run_hook(
+        hook: 'Hook',
+        file_args: Sequence[str],
+        color: bool,
+) -> Tuple[int, bytes]:
     return helpers.run_xargs(hook, hook.cmd, file_args, color=color)
