@@ -1,17 +1,19 @@
-from __future__ import unicode_literals
-
 import argparse
 import logging
 import os
 import sys
+from typing import Any
+from typing import Optional
+from typing import Sequence
+from typing import Union
 
 import pre_commit.constants as C
 from pre_commit import color
-from pre_commit import five
 from pre_commit import git
 from pre_commit.commands.autoupdate import autoupdate
 from pre_commit.commands.clean import clean
 from pre_commit.commands.gc import gc
+from pre_commit.commands.hook_impl import hook_impl
 from pre_commit.commands.init_templatedir import init_templatedir
 from pre_commit.commands.install_uninstall import install
 from pre_commit.commands.install_uninstall import install_hooks
@@ -39,7 +41,7 @@ os.environ.pop('__PYVENV_LAUNCHER__', None)
 COMMANDS_NO_GIT = {'clean', 'gc', 'init-templatedir', 'sample-config'}
 
 
-def _add_color_option(parser):
+def _add_color_option(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         '--color', default=os.environ.get('PRE_COMMIT_COLOR', 'auto'),
         type=color.use_color,
@@ -48,7 +50,7 @@ def _add_color_option(parser):
     )
 
 
-def _add_config_option(parser):
+def _add_config_option(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         '-c', '--config', default=C.CONFIG_FILE,
         help='Path to alternate config file',
@@ -56,18 +58,24 @@ def _add_config_option(parser):
 
 
 class AppendReplaceDefault(argparse.Action):
-    def __init__(self, *args, **kwargs):
-        super(AppendReplaceDefault, self).__init__(*args, **kwargs)
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
         self.appended = False
 
-    def __call__(self, parser, namespace, values, option_string=None):
+    def __call__(
+            self,
+            parser: argparse.ArgumentParser,
+            namespace: argparse.Namespace,
+            values: Union[str, Sequence[str], None],
+            option_string: Optional[str] = None,
+    ) -> None:
         if not self.appended:
             setattr(namespace, self.dest, [])
             self.appended = True
         getattr(namespace, self.dest).append(values)
 
 
-def _add_hook_type_option(parser):
+def _add_hook_type_option(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         '-t', '--hook-type', choices=(
             'pre-commit', 'pre-merge-commit', 'pre-push',
@@ -79,7 +87,7 @@ def _add_hook_type_option(parser):
     )
 
 
-def _add_run_options(parser):
+def _add_run_options(parser: argparse.ArgumentParser) -> None:
     parser.add_argument('hook', nargs='?', help='A single hook-id to run')
     parser.add_argument('--verbose', '-v', action='store_true', default=False)
     parser.add_argument(
@@ -94,6 +102,10 @@ def _add_run_options(parser):
         '--commit-msg-filename',
         help='Filename to check when running during `commit-msg`',
     )
+    parser.add_argument(
+        '--remote-name', help='Remote name used by `git push`.',
+    )
+    parser.add_argument('--remote-url', help='Remote url used by `git push`.')
     parser.add_argument(
         '--hook-stage', choices=C.STAGES, default='commit',
         help='The stage during which the hook is fired.  One of %(choices)s',
@@ -113,7 +125,7 @@ def _add_run_options(parser):
     )
 
 
-def _adjust_args_and_chdir(args):
+def _adjust_args_and_chdir(args: argparse.Namespace) -> None:
     # `--config` was specified relative to the non-root working directory
     if os.path.exists(args.config):
         args.config = os.path.abspath(args.config)
@@ -145,16 +157,15 @@ def _adjust_args_and_chdir(args):
         args.repo = os.path.relpath(args.repo)
 
 
-def main(argv=None):
+def main(argv: Optional[Sequence[str]] = None) -> int:
     argv = argv if argv is not None else sys.argv[1:]
-    argv = [five.to_text(arg) for arg in argv]
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(prog='pre-commit')
 
     # https://stackoverflow.com/a/8521644/812183
     parser.add_argument(
         '-V', '--version',
         action='version',
-        version='%(prog)s {}'.format(C.VERSION),
+        version=f'%(prog)s {C.VERSION}',
     )
 
     subparsers = parser.add_subparsers(dest='command')
@@ -165,9 +176,6 @@ def main(argv=None):
     )
     _add_color_option(autoupdate_parser)
     _add_config_option(autoupdate_parser)
-    autoupdate_parser.add_argument(
-        '--tags-only', action='store_true', help='LEGACY: for compatibility',
-    )
     autoupdate_parser.add_argument(
         '--bleeding-edge', action='store_true',
         help=(
@@ -190,6 +198,16 @@ def main(argv=None):
     _add_color_option(clean_parser)
     _add_config_option(clean_parser)
 
+    hook_impl_parser = subparsers.add_parser('hook-impl')
+    _add_color_option(hook_impl_parser)
+    _add_config_option(hook_impl_parser)
+    hook_impl_parser.add_argument('--hook-type')
+    hook_impl_parser.add_argument('--hook-dir')
+    hook_impl_parser.add_argument(
+        '--skip-on-missing-config', action='store_true',
+    )
+    hook_impl_parser.add_argument(dest='rest', nargs=argparse.REMAINDER)
+
     gc_parser = subparsers.add_parser('gc', help='Clean unused cached repos.')
     _add_color_option(gc_parser)
     _add_config_option(gc_parser)
@@ -257,7 +275,7 @@ def main(argv=None):
     _add_run_options(run_parser)
 
     sample_config_parser = subparsers.add_parser(
-        'sample-config', help='Produce a sample {} file'.format(C.CONFIG_FILE),
+        'sample-config', help=f'Produce a sample {C.CONFIG_FILE} file',
     )
     _add_color_option(sample_config_parser)
     _add_config_option(sample_config_parser)
@@ -312,8 +330,6 @@ def main(argv=None):
         store.mark_config_used(args.config)
 
         if args.command == 'autoupdate':
-            if args.tags_only:
-                logger.warning('--tags-only is the default')
             return autoupdate(
                 args.config, store,
                 tags_only=not args.bleeding_edge,
@@ -324,11 +340,22 @@ def main(argv=None):
             return clean(store)
         elif args.command == 'gc':
             return gc(store)
+        elif args.command == 'hook-impl':
+            return hook_impl(
+                store,
+                config=args.config,
+                color=args.color,
+                hook_type=args.hook_type,
+                hook_dir=args.hook_dir,
+                skip_on_missing_config=args.skip_on_missing_config,
+                args=args.rest[1:],
+            )
         elif args.command == 'install':
             return install(
                 args.config, store,
                 hook_types=args.hook_types,
-                overwrite=args.overwrite, hooks=args.install_hooks,
+                overwrite=args.overwrite,
+                hooks=args.install_hooks,
                 skip_on_missing_config=args.allow_missing_config,
             )
         elif args.command == 'init-templatedir':
@@ -350,11 +377,11 @@ def main(argv=None):
             return uninstall(hook_types=args.hook_types)
         else:
             raise NotImplementedError(
-                'Command {} not implemented.'.format(args.command),
+                f'Command {args.command} not implemented.',
             )
 
         raise AssertionError(
-            'Command {} failed to exit with a returncode'.format(args.command),
+            f'Command {args.command} failed to exit with a returncode',
         )
 
 
